/***************************************************************************//**
*   @file   ad5272.c
*   @brief  Implementation of AD5272 Driver for Microblaze processor.
*   @author DBogdan (dragos.bogdan@analog.com)
*
********************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
********************************************************************************
*   SVN Revision: 468
*******************************************************************************/
#define AD5272_GLOBALS

/******************************************************************************/
/* Include Files															  */
/******************************************************************************/
#include "ad5272.h"
#include "../drivers/i2c.h"
#include "../drivers/gpio.h"

/******************************************************************************/
/*  Private variables                                                         */
/******************************************************************************/
unsigned char oldTpProgramButton		= 0; // Old value of *tpProgramButton.
unsigned char oldRdacWpButton			= 0; // Old value of *rdacWpButton.
unsigned char oldResPerfButton			= 0; // Old value of *resPerfButton.
unsigned char controlRegister			= 0; // Content of the control Register.
unsigned char oldPowerDownButton		= 0; // Old value of *powerDownButton.
unsigned char oldHardResetButton		= 0; // Old value of *hardResetButton.
unsigned char fuseAddress 				= 0; // Value of the last programmed memory location.
unsigned char errorStatus				= 0; // Error status.

/******************************************************************************/
/*  Aliases for the uC Probe data                                             */
/******************************************************************************/
volatile unsigned short* readRdacValue	= &AD5272_data[0].usiData;	// Value of the RDAC register.
volatile unsigned char* readRdacButton	= &AD5272_data[1].uchData;	// Position of the Read RDAC button.
volatile unsigned short* writeRdacValue	= &AD5272_data[2].usiData;	// Position of the Write RDAC slider.
volatile unsigned char* writeRdacButton	= &AD5272_data[3].uchData;	// Position of the Write RDAC button.
volatile unsigned char* tpProgramButton = &AD5272_data[4].uchData;	// Position of the 50-TP Program button.
volatile unsigned char* rdacWpButton	= &AD5272_data[5].uchData;	// Position of the RDAC Write Protect button.
volatile unsigned char* resPerfButton 	= &AD5272_data[6].uchData;	// Position of the Resistor Performance.
volatile unsigned char* softResetButton	= &AD5272_data[7].uchData;	// Position of the Soft Reset button.
volatile unsigned char* hardResetButton	= &AD5272_data[8].uchData;	// Position of the Hard Reset button.
volatile unsigned char* powerDownButton	= &AD5272_data[9].uchData;	// Position of the Power Down button.
volatile unsigned char* blowFuseButton	= &AD5272_data[10].uchData;	// Position of the Blow Fuse button.
volatile unsigned char* fuseContButton	= &AD5272_data[11].uchData;	// Position of the Read Fuse Content button.
volatile unsigned char* fuseAddrButton	= &AD5272_data[12].uchData;	// Position of the Read Fuse Address button.
volatile unsigned short* fuseContValue	= &AD5272_data[13].usiData;	// Value of the Fuse Content.
volatile unsigned short* fuseAddrValue	= &AD5272_data[14].usiData;	// Value of the Fuse Address.

/***************************************************************************//**
 * @brief Reads data from AD5272.
 *
 * @param frame2 - Frame2 content.
 * @param frame3 - Frame3 content.
 *
 * @return readData - Received data.
*******************************************************************************/
unsigned short AD5272_Read(unsigned char frame2, unsigned char frame3)
{
	unsigned short readData = 0;
	unsigned char txBuf[2];
	unsigned char rxBuf[2];
	
	txBuf[0] = frame2;
	txBuf[1] = frame3;
	I2C_Write(XPAR_AXI_IIC_0_BASEADDR, AD5272_ADDRESS, -1, 2, txBuf);
	
	I2C_Read(XPAR_AXI_IIC_0_BASEADDR, AD5272_ADDRESS, -1, 2, rxBuf);
	readData = ((unsigned short)rxBuf[0] << 8) + rxBuf[1];

	return(readData);
}

/***************************************************************************//**
 * @brief Writes data to AD5272.
 *
 * @param command - Desired command.
 * @param data - Data which will be loaded.
 *
 * @return None.
*******************************************************************************/
void AD5272_Write(unsigned char command, unsigned short data)
{
	unsigned char txBuf[2];
	
	txBuf[0] = (command << 2) + ((data & 0x0300) >> 8);
	txBuf[1] = (data & 0x00FF);
	
	I2C_Write(XPAR_AXI_IIC_0_BASEADDR, AD5272_ADDRESS, -1, 2, txBuf);
}

/***************************************************************************//**
 * @brief Initializes I2C and initial values for AD5272 board.
 *
 * @param None.
 *
 * @return Always returns success.
*******************************************************************************/
int AD5272_Init(void)
{
	GPIO_SET_OUTPUT_PINS(XPAR_AXI_GPIO_0_BASEADDR, ~AD5272_RESET_PIN);
	GPIO_SET_BITS(XPAR_AXI_GPIO_0_BASEADDR,	AD5272_RESET_PIN);

	I2C_Init(XPAR_AXI_IIC_0_BASEADDR, AD5272_ADDRESS);	
	
	return(TRUE);
}

/**************************************************************************//**
 * @brief Updates the device's monitored and controlled parameters.
 *
 * @param None.
 *
 * @return Always returns success.
******************************************************************************/
int AD5272_UpdateParameters(void)
{
	if(*readRdacButton)		// If the Read RDAC button was pressed,
	{						// reads content of RDAC wiper register.
		*readRdacValue = AD5272_Read(AD5272_READ_RDAC << 2,0x0);
		*readRdacButton = 0;
	}
	if(*writeRdacButton)	// If the Write RDAC button was pressed,
	{						// writes content of serial register data to RDAC.
		AD5272_Write(AD5272_WRITE_RDAC, *writeRdacValue);
		*writeRdacButton = 0;
	}
	if(*tpProgramButton != oldTpProgramButton)	// If the 50-TP Program button has changed his position,
	{											// enables/disables device for 50-TP program.
		controlRegister = AD5272_Read(AD5272_READ_CONTROL << 2,0x0);
		controlRegister = controlRegister & ~ AD5272_C0;
		controlRegister = controlRegister | (*tpProgramButton * AD5272_C0);
		AD5272_Write(AD5272_WRITE_CONTROL,controlRegister);
		oldTpProgramButton = *tpProgramButton;
	}
	if(*rdacWpButton != oldRdacWpButton)	// If the RDAC Write Protect button has changed his position,
	{										// allows or not update of wiper position through a digital interface.
		controlRegister = AD5272_Read(AD5272_READ_CONTROL << 2,0x0);
		controlRegister = controlRegister & ~ AD5272_C1;
		controlRegister = controlRegister | (*rdacWpButton * AD5272_C1);
		AD5272_Write(AD5272_WRITE_CONTROL,controlRegister);
		oldRdacWpButton = *rdacWpButton;
	}
	if(*resPerfButton != oldResPerfButton)	// If the Resistor Performance button has changed his position,
	{										// enables/disables resistor tolerance calibration.
		controlRegister = AD5272_Read(AD5272_READ_CONTROL << 2,0x0);
		controlRegister = controlRegister & ~ AD5272_C2;
		controlRegister = controlRegister | (*resPerfButton * AD5272_C2);
		AD5272_Write(AD5272_WRITE_CONTROL,controlRegister);
		oldResPerfButton = *resPerfButton;
	}
	if(*softResetButton)						// If the Soft Reset button was pressed,
	{											// sends the Software Reset command.
		AD5272_Write(AD5272_RESET,0x00);
		*softResetButton = 0;
	}
	if(*hardResetButton != oldHardResetButton)	// If the Hard Reset button has changed his position,
	{											// toggle the Reset Pin.
		if(*hardResetButton)
		{
			GPIO_CLEAR_BITS(XPAR_AXI_GPIO_0_BASEADDR, AD5272_RESET_PIN);
		}
		else
		{
			GPIO_SET_BITS(XPAR_AXI_GPIO_0_BASEADDR, AD5272_RESET_PIN);
		}
		oldHardResetButton = *hardResetButton;
	}
	if(*powerDownButton != oldPowerDownButton)	// If the Power Down button has changed his position,
	{											// sends the Software Shutdown command.
		AD5272_Write(AD5272_SHUTDOWN,*powerDownButton);
		oldPowerDownButton = *powerDownButton;
	}
	if(*blowFuseButton)							// If the Blow Fuse button was pressed,
	{											// stores RDAC setting to 50-TP.
		AD5272_Write(AD5272_STORE_RDAC_50TP,0x00);
		*blowFuseButton = 0;
	}
	if(*fuseContButton)			// If the Read Fuse Content button was pressed,
	{							// reads content of 50-TP memory.
		fuseAddress = AD5272_Read(AD5272_READ_ADDRESS_50TP << 2,0x0);
		*fuseContValue = AD5272_Read(AD5272_READ_CONTENT_50TP << 2,fuseAddress);
		*fuseContButton = 0;
	}
	if(*fuseAddrButton)			// If the Read Fuse Address button was pressed,
	{							// reads address of the last 50-TP programmed memory location.
		*fuseAddrValue = AD5272_Read(AD5272_READ_ADDRESS_50TP << 2,0x0);
		*fuseAddrButton = 0;
	}

	return (TRUE);
}

